"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault")["default"];

exports.__esModule = true;
exports.getSeriesStatus = getSeriesStatus;
exports.getDatumStatus = getDatumStatus;
exports.materializeStyles = materializeStyles;
exports.isValidPoint = isValidPoint;
exports.translateX = translateX;
exports.translateY = translateY;
exports.translate = translate;
exports.getSecondaryStart = getSecondaryStart;
exports.getSecondary = getSecondary;
exports.getPrimary = getPrimary;
exports.getPrimaryStart = getPrimaryStart;
exports.getPrimaryLength = getPrimaryLength;
exports.getSecondaryLength = getSecondaryLength;
exports.getX = getX;
exports.getXStart = getXStart;
exports.getY = getY;
exports.getYStart = getYStart;
exports.getRectX = getRectX;
exports.getRectY = getRectY;
exports.getWidth = getWidth;
exports.getHeight = getHeight;
exports.clampPxToAxis = clampPxToAxis;
exports.getTickPx = getTickPx;
exports.sortDatumsBySecondaryPx = sortDatumsBySecondaryPx;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

function getSeriesStatus(series, focusedDatum) {
  if ((focusedDatum == null ? void 0 : focusedDatum.seriesId) === series.id) {
    return 'focused';
  }

  return 'none';
}

function getDatumStatus(datum, focusedDatum) {
  var _datum$tooltipGroup;

  if (datum === focusedDatum) {
    return 'focused';
  }

  if ((_datum$tooltipGroup = datum.tooltipGroup) != null && _datum$tooltipGroup.some(function (groupDatum) {
    groupDatum.seriesId === (focusedDatum == null ? void 0 : focusedDatum.seriesId) && groupDatum.index === (focusedDatum == null ? void 0 : focusedDatum.index);
  })) {
    return 'groupFocused';
  }

  return 'none';
}

function normalizeColor(style, defaults) {
  return (0, _extends2["default"])({}, style, {
    stroke: style.stroke || style.color || defaults.stroke || defaults.color,
    fill: style.fill || style.color || defaults.fill || defaults.color
  });
}

var elementTypes = ['area', 'line', 'rectangle', 'circle'];

function materializeStyles(style, defaults) {
  if (style === void 0) {
    style = {};
  }

  if (defaults === void 0) {
    defaults = {};
  }

  style = normalizeColor(style, defaults);

  for (var i = 0; i < elementTypes.length; i++) {
    var type = elementTypes[i];

    if (style[type] && defaults[type]) {
      style[type] = materializeStyles(style[type], defaults);
    }
  }

  return style;
}

function isValidPoint(d) {
  if (d === null) {
    return false;
  }

  if (typeof d === 'undefined') {
    return false;
  }

  if (typeof d === 'string' && d === 'null') {
    return false;
  }

  return true;
}

function translateX(x) {
  return "translate3d(" + Math.round(x) + "px, 0, 0)";
}

function translateY(y) {
  return "translate3d(0, " + Math.round(y) + "px, 0)";
}

function translate(x, y) {
  return "translate3d(" + Math.round(x) + "px, " + Math.round(y) + "px, 0)";
}

function getSecondaryStart(datum, secondaryAxis) {
  var _secondaryAxis$scale2;

  if (secondaryAxis.stacked) {
    var _secondaryAxis$scale, _datum$stackData$, _datum$stackData;

    return (_secondaryAxis$scale = secondaryAxis.scale((_datum$stackData$ = (_datum$stackData = datum.stackData) == null ? void 0 : _datum$stackData[0]) != null ? _datum$stackData$ : NaN)) != null ? _secondaryAxis$scale : NaN;
  }

  return (_secondaryAxis$scale2 = secondaryAxis.scale(0)) != null ? _secondaryAxis$scale2 : NaN;
}

function getSecondary(datum, secondaryAxis) {
  var _secondaryAxis$scale4;

  if (secondaryAxis.stacked) {
    var _secondaryAxis$scale3, _datum$stackData$2, _datum$stackData2;

    return (_secondaryAxis$scale3 = secondaryAxis.scale((_datum$stackData$2 = (_datum$stackData2 = datum.stackData) == null ? void 0 : _datum$stackData2[1]) != null ? _datum$stackData$2 : NaN)) != null ? _secondaryAxis$scale3 : NaN;
  }

  return (_secondaryAxis$scale4 = secondaryAxis.scale(datum.secondaryValue)) != null ? _secondaryAxis$scale4 : NaN;
}

function getPrimary(datum, primaryAxis) {
  var primary;

  if (primaryAxis.stacked) {
    var _primaryAxis$scale, _datum$stackData$3, _datum$stackData3;

    primary = (_primaryAxis$scale = primaryAxis.scale((_datum$stackData$3 = (_datum$stackData3 = datum.stackData) == null ? void 0 : _datum$stackData3[1]) != null ? _datum$stackData$3 : NaN)) != null ? _primaryAxis$scale : NaN;
  } else {
    var _primaryAxis$scale2;

    primary = (_primaryAxis$scale2 = primaryAxis.scale(datum.primaryValue)) != null ? _primaryAxis$scale2 : NaN;
  }

  if (primaryAxis.axisFamily === 'band') {
    primary = primary + getPrimaryLength(datum, primaryAxis) / 2;
  }

  return primary;
}

function getPrimaryStart(datum, primaryAxis) {
  return getPrimary(datum, primaryAxis) - getPrimaryLength(datum, primaryAxis) / 2;
}

function getPrimaryLength(_datum, primaryAxis) {
  if (primaryAxis.axisFamily === 'band') {
    var _primaryAxis$minBandS, _primaryAxis$maxBandS;

    return Math.min(Math.max(primaryAxis.scale.bandwidth(), (_primaryAxis$minBandS = primaryAxis.minBandSize) != null ? _primaryAxis$minBandS : 1), (_primaryAxis$maxBandS = primaryAxis.maxBandSize) != null ? _primaryAxis$maxBandS : 99999999);
  }

  return Math.max(primaryAxis.bandScale.bandwidth(), 1);
}

function getSecondaryLength(datum, secondaryAxis) {
  var secondary = [getSecondaryStart(datum, secondaryAxis), getSecondary(datum, secondaryAxis)];
  return Math.abs(secondary[1] - secondary[0]);
}

function getX(datum, primaryAxis, secondaryAxis) {
  return primaryAxis.isVertical ? getSecondary(datum, secondaryAxis) : getPrimary(datum, primaryAxis);
}

function getXStart(datum, primaryAxis, secondaryAxis) {
  return primaryAxis.isVertical ? getSecondaryStart(datum, secondaryAxis) : getPrimaryStart(datum, primaryAxis);
}

function getY(datum, primaryAxis, secondaryAxis) {
  return primaryAxis.isVertical ? getPrimary(datum, primaryAxis) : getSecondary(datum, secondaryAxis);
}

function getYStart(datum, primaryAxis, secondaryAxis) {
  return primaryAxis.isVertical ? getPrimaryStart(datum, primaryAxis) : getSecondaryStart(datum, secondaryAxis);
}

function getRectX(datum, primaryAxis, secondaryAxis) {
  return primaryAxis.isVertical ? getSecondaryStart(datum, secondaryAxis) : getPrimaryStart(datum, primaryAxis);
}

function getRectY(datum, primaryAxis, secondaryAxis) {
  return primaryAxis.isVertical ? getPrimaryStart(datum, primaryAxis) : getSecondary(datum, secondaryAxis);
}

function getWidth(datum, primaryAxis, secondaryAxis) {
  return primaryAxis.isVertical ? getSecondaryLength(datum, secondaryAxis) : getPrimaryLength(datum, primaryAxis);
}

function getHeight(datum, primaryAxis, secondaryAxis) {
  return primaryAxis.isVertical ? getPrimaryLength(datum, primaryAxis) : getSecondaryLength(datum, secondaryAxis);
}

function clampPxToAxis(px, axis) {
  var range = axis.scale.range();

  if (axis.isVertical) {
    range.reverse();
  }

  return Math.max(range[0], Math.min(px, range[1]));
}

function getTickPx(scale, value) {
  var _scale;

  var px = (_scale = scale(value)) != null ? _scale : NaN; // @ts-ignore

  if (scale.bandwidth) {
    // @ts-ignore
    return px + scale.bandwidth() / 2;
  }

  return px;
}

function sortDatumsBySecondaryPx(datums, secondaryAxes) {
  return [].concat(datums).sort(function (a, b) {
    var _aAxis$scale, _a$stackData, _bAxis$scale, _b$stackData;

    var aAxis = secondaryAxes.find(function (d) {
      return d.id === a.secondaryAxisId;
    });
    var bAxis = secondaryAxes.find(function (d) {
      return d.id === b.secondaryAxisId;
    });
    var aPx = (_aAxis$scale = aAxis == null ? void 0 : aAxis.scale(aAxis.stacked ? (_a$stackData = a.stackData) == null ? void 0 : _a$stackData[1] : a.secondaryValue)) != null ? _aAxis$scale : NaN;
    var bPx = (_bAxis$scale = bAxis == null ? void 0 : bAxis.scale(bAxis.stacked ? (_b$stackData = b.stackData) == null ? void 0 : _b$stackData[1] : b.secondaryValue)) != null ? _bAxis$scale : NaN;
    return aPx - bPx;
  });
}